###############################################################################
#	$VER: html-object.wl 2.2 (30.10.2000)
#
#	Copyright  1997,1998,2000 Udo Schuermann
#	All rights reserved.
#
#	This file is part of the WebLord software distribution and is required
#	if you use the old IMAGE object type, which is no longer directly
#	supported as of version 2.0
#
#	WebLord performs an internal conversion of IMAGE objects when these
#	are encountered. The process involves retyping the object as TEXT,
#	adding a VALUE property with a value of "html-image" (i.e. the object
#	defined in this file!), and changing all the object's other property
#	names to begin with "image-" (i.e. "link" becomes "image-link")
#
#	The HTML-IMAGE object defined in this file makes the conversion as
#	smooth and painless as possible, thereby freeing WebLord from the
#	burdens of an implementation that is too closely tied to HTML. I.e.
#	WebLord will be able to easily move into the realm of XML (or apply
#	itself to any other language that you may wish to use.)
#
#	NOTE: Rather than continuing to define IMAGE objects (and suffering
#	the performance penalties of the internal conversion) you should use
#	either TEXT objects directly, or define your images in the following
#	manner:
#
#		text = my-image
#		{
#			value =		html-object;
#			object-data =	"pix/test.gif";
#			object-width =	"128";
#			object-height =	"72";
#			object-align =	"left";
#			object-border =	"0";
#			object-alt =	"A test image";
#			object-target =	"window-id";
#
#		# either one of the following are acceptable *in this case*
#		# but the first ('link') is preferred:
#			link =		"http://my.site/pix/test-big.gif";
#		}
#
#	Then merely reference the object the way you would reference
#	an IMAGE object:    "This is a test: " my-image
#	Omit any items you don't need. 'image-src' is the only property that
#	is required.
#
#	Notice that it is easy to adapt the code below for the HTML 4.0 general
#	purpose OBJECT element to handle images, applets, animations, movies,
#	etc. in a more flexible way.
#
###############################################################################
# This is where the good stuff starts. Good study material, too! :-)
# Note that the incessant use of ^ and _ symbols is meant to keep the object
# names "out of your way": they have no special meaning.
#

#requires-version 2.2

text = html-object
{
	if    = isdef object-data;	# did the conversion process produce an 'object-data' property?
	then  = ^html-object-center-start
			^html-object-link-start
				"<object" ^html-object-spec ">"
				^html-object-nest
				"</object>"
			^html-object-link-end
		^html-object-center-end;
	else  = html-object-error( text = "Object '" parent.objectname "' defines no 'object-data' property; "
				  "you may need to convert this manually. The object is defined thus:\n"
				  parent.objectdefinition );
}

text html-object-error
{
	echo	= error++ text;		# the 'error++' increments the error counter; echo property is echoed to output no
	value	= "";			# value to be returned in this case; still need to define it to prevent a complaint
}

text = ^html-object-spec
{
	value = ^html-object-data
		^html-object-width
		^html-object-height
		^html-object-type
		^html-object-align
		^html-object-alt;
}

text = ^html-object-nest
{
	if = object-nest "=" "";
	then = ^object-image ;
	else = object-nest;
}
text = ^object-image
{
	value =	html-image;
	image-src =	object-data;
	image-width =	object-width;
	image-height =	object-height;
	image-border =	object-border;
	image-alt =	object-alt;
	image-align =	'';
	image-link =	'';
}

text = ^html-object-center-start	{ if = object-align "=" "center";	then = "<div align=" HTML-Q(value="center") ">" }
text = ^html-object-center-end	{ if = object-align "=" "center";	then = "</div>" }

text = ^html-object-data	{ 					value = " data=" HTML-Q( value = object-data ) }
text = ^html-object-type	{ if = object-type "=" "";		then = ""; else = " type=" HTML-Q(value=object-type) }

text = ^html-object-width 	{ if = isdef object-width;		then = " width=" HTML-Q( value = object-width ) }
text = ^html-object-height	{ if = isdef object-height;		then = " height=" HTML-Q( value = object-height ) }

text = ^html-object-align	{ if = isdef object-align "&" object-align "!=" "center";
				  then = " align=" HTML-Q( value = object-align ) }

# The "border=..." is added to an image only if it is one that has an anchor/link specified. Lovely optimization, eh? :-)
text = ^html-object-border	{ if = isdef object-border "and" "[" isdef link "or" isdef object-link "]";
				  then = " border=" HTML-Q( value = object-border ) }

text = ^html-object-alt		{ if = isdef object-alt;			then = " alt=" HTML-Q( value = object-alt ) }

#
# The following is used only an old IMAGE objects that have been converted to TEXT objects internally.
# The conversion process causes "object-" to be prepended to each property, wherefore the internal logic
# will not recognize "object-link" as a standard property; we catch this here:
#
text = ^html-object-link-start	{ if = isdef object-link;	then = "<a" ^_html-object-link-to ^_html-object-target-to ">" }
text = ^html-object-link-end	{ if = isdef object-link;	then = "</a>" }
text = ^_html-object-link-to	{ 				value = " href=" HTML-Q(value=object-link) }
text = ^_html-object-target-to	{ if = isdef object-target;	then = " target=" HTML-Q(value=object-target) }

#end-of-file :-)
